@@ -38,9 +38,6 @@ class ReadBed(object):
         else:
             self.get_bed_interval(fields, is_first_line=True)
         self.file_handle.seek(0)
-        self.prev_chrom = None
-        self.prev_start = -1
-        self.prev_line = None
 
         # list of bed fields
         self.fields = ['chromosome', 'start', 'end',
@@ -74,25 +71,6 @@ class ReadBed(object):
         self.line_number += 1
         return line
 
-    def next(self):
-        """
-        :return: bedInterval object
-        """
-        line = self.get_no_comment_line()
-
-        bed = self.get_bed_interval(line)
-        if self.prev_chrom == bed.chromosome:
-            assert self.prev_start <= bed.start, \
-                "Bed file not sorted. Please use a sorted bed file.\n" \
-                "File: {}\n" \
-                "Previous line: {}\n Current line{} ".format(self.file_handle.name, self.prev_line, line)
-
-        self.prev_chrom = bed.chromosome
-        self.prev_start = bed.start
-        self.prev_line = line
-
-        return bed
-
     def __next__(self):
         """
         :return: bedInterval object
@@ -100,15 +78,6 @@ class ReadBed(object):
         line = self.get_no_comment_line()
 
         bed = self.get_bed_interval(line)
-        if self.prev_chrom == bed.chromosome:
-            assert self.prev_start <= bed.start, \
-                "Bed file not sorted. Please use a sorted bed file.\n" \
-                "File: {}\n" \
-                "Previous line: {}\n Current line{} ".format(self.file_handle.name, self.prev_line, line)
-
-        self.prev_chrom = bed.chromosome
-        self.prev_start = bed.start
-        self.prev_line = line
 
         return bed
 
@@ -117,7 +86,7 @@ class ReadBed(object):
         Processes each bed line from a bed file, casts the values and returns
         a namedtuple object
 
-        >>> bed_line="chr1\t0\t1000\tgene_1\t0.5\t-\t0\t1000\t0\t3\t10,20,100\t20,200,700"
+        >>> bed_line="chr1\t0\t1000\tgene_1\t0.5\t-\t0\t1000\t0\t3\t10,20,300\t0,200,700"
         >>> with open('/tmp/test.bed', 'w') as fh:
         ...     foo = fh.write(bed_line)
         >>> bed_f = ReadBed(open('/tmp/test.bed','r'))
@@ -125,7 +94,7 @@ class ReadBed(object):
         >>> bed.chromosome
         'chr1'
         >>> bed.block_starts
-        [20, 200, 700]
+        [0, 200, 700]
 
         >>> bed_line="chr2\t0\t1000\tgene_1\t0.5\t-\n"
         >>> with open('/tmp/test.bed', 'w') as fh:
@@ -138,17 +107,20 @@ class ReadBed(object):
         line_data = bed_line.strip()
         line_data = to_string(line_data)
         line_data = line_data.split("\t")
+        if is_first_line:
+            assert len(line_data) != 1, \
+                "Only one field detected, you may use" \
+                " a bed delimited by space. This format " \
+                "is not supported by pyGenomeTracks."
 
         if not is_first_line:
             if self.file_type != 'bed6':
                 # When bed6 you can have less fields in one row
                 # because there are default values
                 assert len(line_data) >= self.fields_to_read, \
-                    "File type detected is {} but line {}: {} does " \
-                    "not have {} fields.".format(self.file_type,
-                                                 self.line_number,
-                                                 bed_line,
-                                                 self.fields_to_read)
+                    f"File type detected is {self.file_type} but line" \
+                    f" {self.line_number}: {bed_line} does " \
+                    f"not have {self.fields_to_read} fields."
             line_data = line_data[:self.fields_to_read]
 
         line_values = []
@@ -167,10 +139,9 @@ class ReadBed(object):
                         r = '-'
                     else:
                         sys.stderr.write("*Warning, invalid strand value"
-                                         "found {} for line #{}:\n{}\n "
+                                         f"found {r} for line #{bed_line}:\n{self.line_number}\n "
                                          "Setting strand to '.'"
-                                         "\n".format(r, bed_line,
-                                                     self.line_number))
+                                         "\n")
                         r = '.'
                 line_values.append(r)
 
@@ -179,43 +150,66 @@ class ReadBed(object):
                 try:
                     line_values.append(int(r))
                 except ValueError:
-                    raise InputError("Value: {} in field {} at line {}"
+                    raise InputError(f"Value: {r} in field {idx + 1} at line"
+                                     f" {self.line_number}"
                                      " is not an integer. This is "
                                      "probably not a bed file."
-                                     "\n".format(r, idx + 1,
-                                                 self.line_number))
-            elif idx in [6, 7, 9]:
-                # thichStart(6), thickEnd(7) and blockCount(9) fields
+                                     "\n")
+            elif idx in [6, 7]:
+                # thichStart(6) and thickEnd(7) fields
                 # Should be integer, if they are not we change the bed type
                 try:
                     line_values.append(int(r))
                 except ValueError:
                     if is_first_line:
-                        if idx == 9:
-                            self.file_type = 'bed8'
-                            self.fields_to_read = 8
-                        else:
-                            self.file_type = 'bed6'
-                            self.fields_to_read = 6
-                        sys.stderr.write("Value: {} in field {}"
+                        self.file_type = 'bed6'
+                        self.fields_to_read = 6
+                        sys.stderr.write(f"Value: {r} in field {idx + 1}"
                                          " is not an integer"
-                                         "\n Only the first {} fields will"
-                                         " be used.\n".format(r, idx + 1,
-                                                              self.fields_to_read))
+                                         "\n Only the first "
+                                         f"{self.fields_to_read} fields will"
+                                         " be used.\n")
                         break
                     else:
-                        default_value = 0 if (idx == 9) else line_values[1]
-                        sys.stderr.write("Value: {} in field {} at line {}"
+                        if idx == 6:
+                            default_value = line_values[1]
+                        else:  # idx is 7
+                            default_value = line_values[6]
+                        line_values.append(default_value)
+                        sys.stderr.write(f"Value: {r} in field {idx + 1} at"
+                                         f" line {self.line_number}"
                                          " is not an integer"
-                                         "\n {} will be used.\n"
-                                         "".format(r, idx + 1,
-                                                   self.line_number,
-                                                   default_value))
+                                         f"\n {default_value} will be used.\n")
+            elif idx == 9:
+                # blockCount(9) field
+                # Should be integer, if they are not we change the bed type
+                try:
+                    line_values.append(int(r))
+                except ValueError:
+                    if is_first_line:
+                        self.file_type = 'bed8'
+                        self.fields_to_read = 8
+                        sys.stderr.write(f"Value: {r} in field {idx + 1}"
+                                         " is not an integer"
+                                         "\n Only the first "
+                                         f"{self.fields_to_read} fields will"
+                                         " be used.\n")
+                        break
+                    else:
+                        sys.stderr.write("Warning: reading line "
+                                         f"#{self.line_number}, "
+                                         f"the block number {r} is not "
+                                         "valid.\nNo block will be used.\n")
+                        line_values.append(1)
+                        line_values.append([line_values[2] - line_values[1]])
+                        line_values.append([0])
+                        break
             # check item rgb
             elif idx == 8:
                 passed = True
                 try:
-                    line_values.append(int(r))
+                    # This is what happens in UCSC browser:
+                    line_values.append([0, 0, int(r)])
                 except ValueError:
                     r = to_string(r)
                     rgb = r.split(",")
@@ -231,20 +225,19 @@ class ReadBed(object):
                 if not passed:
                     if is_first_line:
                         sys.stderr.write("Warning: "
-                                         "The rgb field {} is not "
+                                         f"The rgb field {r} is not "
                                          "valid.\nOnly the first 8 fields"
-                                         " will be used.\n"
-                                         "".format(self.line_number, r))
+                                         " will be used.\n")
                         self.file_type = 'bed8'
                         self.fields_to_read = 8
                         break
                     else:
-                        sys.stderr.write("Warning: reading line: #{}. "
-                                         "The rgb field {} is not "
-                                         "valid.\n0"
-                                         " will be used.\n"
-                                         "".format(self.line_number, r))
-                        line_values.append(0)
+                        sys.stderr.write("Warning: reading line: "
+                                         f"#{self.line_number}. "
+                                         f"The rgb field {r} is not "
+                                         "valid.\n0,0,0"
+                                         " will be used.\n")
+                        line_values.append([0, 0, 0])
 
             elif idx in [10, 11]:
                 # this are the block sizes and block start positions
@@ -255,25 +248,23 @@ class ReadBed(object):
                 except ValueError as detail:
                     if is_first_line:
                         sys.stderr.write("Warning: "
-                                         "The block field {} is not "
-                                         "valid.\nError message: {}"
+                                         f"The block field {r} is not "
+                                         f"valid.\nError message: {detail}"
                                          "\nOnly the first 9 fields"
-                                         " will be used.\n"
-                                         "".format(r,
-                                                   detail))
+                                         " will be used.\n")
                         self.file_type = 'bed9'
                         self.fields_to_read = 9
                         break
                     else:
-                        sys.stderr.write("Warning: reading line #{}, "
-                                         "the block field {} is not "
-                                         "valid.\nError message: {}"
-                                         "\nNo block will be used.\n"
-                                         "".format(self.line_number, r,
-                                                   detail))
-                        line_values[9] = 1
-                        line_values[10] = line_values[1]
-                        line_values[11] = line_values[2] - line_values[1]
+                        sys.stderr.write("Warning: reading line "
+                                         f"#{self.line_number}, "
+                                         f"the block field {r} is not "
+                                         f"valid.\nError message: {detail}"
+                                         "\nOne block will be used.\n")
+                        line_values = line_values[:9]
+                        line_values.append(1)
+                        line_values.append([line_values[2] - line_values[1]])
+                        line_values.append([0])
                         break
                 else:
                     line_values.append(r)
@@ -285,20 +276,18 @@ class ReadBed(object):
                     if is_first_line:
                         sys.stderr.write("Warning: "
                                          "The block field 5 (score) is not "
-                                         "valid: {}.\nError message: {}"
+                                         f"valid: {r}.\nError message: {detail}"
                                          "\nOnly the first 4 fields "
-                                         "will be used.\n".format(r,
-                                                                  detail))
+                                         "will be used.\n")
                         self.file_type = 'bed6'
                         self.fields_to_read = 4
                         break
                     else:
-                        sys.stderr.write("Warning: reading line #{}, "
+                        sys.stderr.write("Warning: reading line "
+                                         f"#{self.line_number}, "
                                          "the block field 5 (score) is not "
-                                         "valid: {}.\nError message: {}"
-                                         "\n0 will be used.\n"
-                                         "".format(self.line_number, r,
-                                                   detail))
+                                         f"valid: {r}.\nError message: {detail}"
+                                         "\n0 will be used.\n")
                         line_values.append(0.)
                 else:
                     line_values.append(tmp)
@@ -309,13 +298,13 @@ class ReadBed(object):
                 if self.fields_to_read <= 6:
                     self.file_type = 'bed6'
                 else:
-                    self.file_type = 'bed{}'.format(self.fields_to_read)
+                    self.file_type = f'bed{self.fields_to_read}'
             return()
 
         assert line_values[2] > line_values[1], \
             "Start position larger or equal than end" \
-            " for line #{}:\n{}\n".format(self.line_number,
-                                          bed_line)
+            f" for line #{self.line_number}:\n{bed_line}\n"
+
         if len(line_values) == 12:
             check_bed12(line_values, self.line_number, bed_line)
 
@@ -323,9 +312,8 @@ class ReadBed(object):
             assert len(line_values) > 2, \
                 "The number of field is less than 3.\n" \
                 "This is not a bed file.\n" \
-                "File: {}\n" \
-                "Current line: {}\n".format(self.file_handle.name,
-                                            line_values)
+                f"File: {self.file_handle.name}\n" \
+                f"Current line: {line_values}\n"
             # If there is less than 6 fields, the default values will be added
             default = [".", 0, "."]
             line_values = [line_values[i] if i < len(line_values)
@@ -340,26 +328,32 @@ def check_bed12(line_values, line_number, bed_line):
     block_sizes = line_values[10]
     block_relative_starts = line_values[11]
     assert len(block_sizes) == block_counts, \
-        "The number of blocks: {} does not correspond to" \
-        "the number of blocks sizes: {}\nline #{}:\n" \
-        "{}".format(block_counts, str(block_sizes),
-                    line_number, bed_line)
+        f"The number of blocks: {block_counts} does not correspond to" \
+        f"the number of blocks sizes: {str(block_sizes)}\nline #{line_number}:" \
+        f"\n{bed_line}"
     assert len(block_relative_starts) == block_counts, \
-        "The number of blocks: {} does not correspond to" \
-        "the number of blocks relative starts: {}\nline #{}:\n" \
-        "{}".format(block_counts, str(block_relative_starts),
-                    line_number, bed_line)
+        f"The number of blocks: {block_counts} does not correspond to" \
+        f"the number of blocks relative starts: {str(block_relative_starts)}" \
+        f"\nline #{line_number}:\n{bed_line}"
     for i in range(block_counts):
         block_start = line_values[1] + block_relative_starts[i]
         block_end = block_start + block_sizes[i]
         assert block_start <= line_values[2], \
-            "The block number {} of line {} has a starting position " \
-            "greater than the end of the feature:\n{}The" \
+            f"The block number {i} of line {line_number} has a starting" \
+            f" position greater than the end of the feature:\n{bed_line}The" \
             " 12th field of a bed12 should contains relative start" \
-            " positions of blocks.".format(i, line_number, bed_line)
+            " positions of blocks."
         assert block_end <= line_values[2], \
-            "The block number {} of line {} has an ending position " \
-            "greater than the end of the feature:\n{}The" \
+            f"The block number {i} of line {line_number} has an ending position " \
+            f"greater than the end of the feature:\n{bed_line}The" \
             " 12th field of a bed12 should contains relative start" \
             " positions of blocks and the 11th field should contains" \
-            " the length of each block.".format(i, line_number, bed_line)
+            " the length of each block."
+    assert min(block_relative_starts) == 0, \
+        f"The blocks relative_starts of line\n{bed_line}\n" \
+        "does not contain 0. BED blocks must span chromStart" \
+        " to chromEnd."
+    assert max([bstart + bsize for bstart, bsize in zip(block_relative_starts, block_sizes)]) == line_values[2] - line_values[1], \
+        f"The blocks described in line\n{bed_line}\n" \
+        "does not cover chromEnd. BED blocks must span chromStart" \
+        " to chromEnd."
