@@ -1,10 +1,20 @@
 # -*- coding: utf-8 -*-
 
-from .. utilities import to_string, to_bytes
+from .. utilities import to_string, to_bytes, InputError, transform
 import logging
 import numpy as np
 from matplotlib import colors as mc
 import matplotlib.pyplot as plt
+from matplotlib.ticker import LogFormatter
+import re
+
+# This is a regex for float which would work for 11, 102.25, but also .2
+float_regex = r'(?:\d+)?(?:\.\d+)?'
+# This is a regex for color_tuple without space: (float,float,float) which
+# put each float in a group:
+color_tuple = re.compile(r'^\(({0}),({0}),({0})\)$'.format(float_regex))
+# This is a regex for group without comma except between parenthesis
+block_no_comma_outside_parenthesis = re.compile(r'(?:[^,(]|\([^)]*\))+')
 
 
 class GenomeTrack(object):
@@ -289,35 +299,48 @@ height = 2
             return False
         if mc.is_color_like(self.properties[param]):
             valid_color = self.properties[param]
-        # It can be a tuple (for example (1, 0.88, 2./3) would be a valid color):
+        # It can be a tuple (for example (1, 0.88, 0.66666) would be a valid color):
+        # Warning: (1, 0.88, 2./3) is not a valid color anymore
         elif self.properties[param][0] == '(':
-            try:
-                custom_color = eval(self.properties[param])
-            except (SyntaxError, NameError) as e:
-                self.log.warning(f"*WARNING*: '{param}' for section"
-                                 f" {self.properties[param]}"
-                                 f" raised an error:\n{e}\n"
-                                 f"{self.properties['section_name']} has "
-                                 "been set to "
-                                 f"{default_value}.\n")
-                valid_color = default_value
-            else:
-                if mc.is_color_like(custom_color):
-                    valid_color = custom_color
-                else:
+            match = color_tuple.match(self.properties[param].replace(' ', ''))
+            if match is not None:
+                try:
+                    custom_color = tuple([float(v) for v in match.groups()])
+                except ValueError as e:
                     self.log.warning(f"*WARNING*: '{param}' for section"
-                                     f" {self.properties[param]}"
-                                     " is not valid. "
-                                     f"{self.properties['section_name']} has "
+                                     f" {self.properties['section_name']}"
+                                     f" raised an error:\n{e}\n"
+                                     f"{param} has "
                                      "been set to "
                                      f"{default_value}.\n")
                     valid_color = default_value
+                else:
+                    if mc.is_color_like(custom_color):
+                        valid_color = custom_color
+                    else:
+                        self.log.warning(f"*WARNING*: '{param}' for section"
+                                         f" {self.properties['section_name']}"
+                                         " is not valid. "
+                                         f"{param} has "
+                                         "been set to "
+                                         f"{default_value}.\n")
+                        valid_color = default_value
+            else:
+                self.log.warning(f"*WARNING*: '{param}' for section"
+                                 f" {self.properties['section_name']}"
+                                 " is not well formatted (expected (r, g, b), "
+                                 "with r,g,b values as float). "
+                                 f"{param} has "
+                                 "been set to "
+                                 f"{default_value}.\n")
+                valid_color = default_value
+
         if not colormap_possible:
             if valid_color is None:
                 self.log.warning(f"*WARNING*: '{param}' for section"
-                                 f" {self.properties[param]}"
+                                 f" {self.properties['section_name']}"
                                  " is not valid. "
-                                 f"{self.properties['section_name']} has "
+                                 f"{param} has "
                                  "been set to "
                                  f"{default_value}.\n")
                 valid_color = default_value
@@ -325,45 +348,54 @@ height = 2
             return False
         else:
             valid_colormap = None
+            message = ""
             # We will try to process the color as a colormap
-            if valid_color is None:
-                # If someone what to use its own colormap,
-                # he can specify the rgb values or color values:
-                # For example:
-                # colormap = ['white', (1, 0.88, 2./3), (1, 0.74, 0.25), (1, 0.5, 0), (1, 0.19, 0), (0.74, 0, 0), (0.35, 0, 0)]
-                if self.properties[param][0] == '[':
-                    try:
-                        custom_colors = eval(self.properties[param])
-                    except (SyntaxError, NameError) as e:
-                        self.log.warning("Warning: section "
-                                         f"{self.properties['section_name']},"
-                                         f" {param} was set as "
-                                         f"{self.properties[param]} but "
-                                         f"raises an error:\n{e}\nIt will be "
-                                         "ignored and default value will be "
-                                         "used.\n")
+            # If someone what to use its own colormap,
+            # he can specify the rgb values or color values:
+            # For example:
+            # colormap = ['white', (1, 0.88, 0.66), (1, 0.74, 0.25), (1, 0.5, 0), (1, 0.19, 0), (0.74, 0, 0), (0.35, 0, 0)]
+            # Warning:
+            # colormap = ['white', (1, 0.88, 2./3), (1, 0.74, 0.25), (1, 0.5, 0), (1, 0.19, 0), (0.74, 0, 0), (0.35, 0, 0)]
+            # is not any more a valid color because of 2./3
+            if self.properties[param][0] == '[':
+                if self.properties[param][-1] == ']':
+                    # We remove all space and quote
+                    prepared_color_list = re.sub(" |\'|\"", "", self.properties[param][1:-1])
+                    # We extract individual colors
+                    match = block_no_comma_outside_parenthesis.findall(prepared_color_list)
+                    if match is not None:
+                        # We check the color which start with '(' are (r,g,b) with rgb as floats
+                        if all([color_tuple.match(v) is not None
+                                for v in match if v[0] == '(']):
+                            # We try to convert them as float:
+                            try:
+                                custom_colors = [tuple([float(v) for v in color_tuple.match(v).groups()])
+                                                 if v[0] == '(' else v for v in match]
+                            except ValueError:
+                                message = "some (r,g,b) values of the list could not be converted to float"
+                            else:
+                                try:
+                                    valid_colormap = mc.LinearSegmentedColormap.from_list(
+                                        'custom', custom_colors, N=100)
+                                except ValueError:
+                                    message = "the list of color could not be converted to colormap"
+                        else:
+                            message = "there is no color between brackets"
                     else:
-                        try:
-                            valid_colormap = mc.LinearSegmentedColormap.from_list(
-                                'custom', custom_colors, N=100)
-                        except ValueError as e:
-                            self.log.warning("Warning: section "
-                                             f"{self.properties['section_name']},"
-                                             f" {param} was set as "
-                                             f"{self.properties[param]} but "
-                                             f"raises an error:\n{e}\nIt will "
-                                             f"be ignored and"
-                                             " default value will be used.\n")
-                else:
-                    if self.properties[param] in dir(plt.cm):
-                        valid_colormap = self.properties[param]
+                        message = "some colors starting with ( in the color" \
+                                  " list are not formatted (r,g,b) with r,g,b as float"
+                if message != "":
+                    message = f" ({message})"
+            else:
+                if self.properties[param] in dir(plt.cm):
+                    valid_colormap = self.properties[param]
         # Here, colormap is possible
         # valid_color is None or a valid color or the default value
         # valid_colormap is None or a valid colormap
         if valid_color is None and valid_colormap is None:
             self.log.warning(f"*WARNING* {param}: '{self.properties[param]}'"
                              f" for section {self.properties['section_name']}"
-                             " is not valid. It has "
+                             f" is not valid{message}. It has "
                              "been set to "
                              f"{default_value}.\n")
             self.properties[param] = default_value
@@ -374,7 +406,7 @@ height = 2
                 self.log.warning(f"*WARNING* {param}: "
                                  f"'{self.properties[param]}' for section"
                                  f" {self.properties['section_name']}"
-                                 " is not valid. It has "
+                                 f" is not valid{message}. It has "
                                  "been set to "
                                  f"{default_value}.\n")
                 valid_colormap = default_value
@@ -388,6 +420,23 @@ height = 2
                 self.properties[param] = valid_colormap
                 return True
 
+    def checkoperation(self):
+        "Will check if the operation is 'safe'"
+        allowed_words = ['second_file', 'file',
+                         'sum', 'min', 'max',
+                         'log1p', 'log']
+        allowed_signs = ['+', '-', '*', '/',
+                         '(', ')', '.', ',', ' ', 'e'] + \
+                        [f"{i}" for i in range(10)]
+        operation = self.properties['operation']
+        for word in allowed_words:
+            operation = operation.replace(word, "")
+        forbidden_signs = [s for s in operation if s not in allowed_signs]
+        if len(forbidden_signs) > 0:
+            raise InputError(f"operation: {self.properties['operation']}"
+                             " uses signs which are not allowed: "
+                             f"{','.join(forbidden_signs)}.")
+
     @staticmethod
     def check_chrom_str_bytes(iteratable_obj, p_obj):
         # determine type
@@ -402,5 +451,71 @@ height = 2
                 p_obj = to_bytes(p_obj)
         return p_obj
 
+    def plot_custom_cobar(self, axis, fraction=0.95):
+        if self.properties.get('transform', 'no') in ['log', 'log1p']:
+            # get a useful log scale
+            # that looks like [1, 2, 5, 10, 20, 50, 100, ... etc]
+
+            formatter = LogFormatter(10, labelOnlyBase=False)
+            aa = np.array([1, 2, 5])
+            tick_values = np.concatenate([aa * 10 ** x for x in range(10)])
+            try:
+                cobar = plt.colorbar(self.img, ticks=tick_values,
+                                     format=formatter, ax=axis,
+                                     fraction=fraction)
+            except AttributeError:
+                return
+        else:
+            try:
+                cobar = plt.colorbar(self.img, ax=axis, fraction=fraction)
+            except AttributeError:
+                try:
+                    cobar = plt.colorbar(self.colormap, ax=axis, fraction=fraction)
+                except AttributeError:
+                    return
+
+        cobar.solids.set_edgecolor("face")
+        cobar.ax.tick_params(labelsize='smaller')
+        cobar.ax.yaxis.set_ticks_position('left')
+        # adjust the labels of the colorbar
+        # Get ticks positions
+        ticks = cobar.ax.get_yticks()
+        # Fix them
+        cobar.set_ticks(ticks)
+        # Set the corresponding labels
+        labels = cobar.ax.set_yticklabels(ticks.astype('float32'))
+        (vmin, vmax) = cobar.mappable.get_clim()
+        for idx in np.where(ticks == vmin)[0]:
+            # if the label is at the start of the colobar
+            # move it above avoid being cut or overlapping with other track
+            labels[idx].set_verticalalignment('bottom')
+        for idx in np.where(ticks == vmax)[0]:
+            # if the label is at the end of the colobar
+            # move it a bit inside to avoid overlapping
+            # with other labels
+            labels[idx].set_verticalalignment('top')
+
+    def adjust_ylim(self, axis):
+        ymax = self.properties.get('max_value', None)
+        ymin = self.properties.get('min_value', None)
+        plot_ymin, plot_ymax = axis.get_ylim()
+        if ymax is None:
+            ymax = plot_ymax
+        else:
+            ymax = transform(np.array([ymax]), self.properties.get('transform', 'no'),
+                             self.properties.get('log_pseudocount', 0),
+                             'ymax')[0]
+        if ymin is None:
+            ymin = plot_ymin
+        else:
+            ymin = transform(np.array([ymin]), self.properties.get('transform', 'no'),
+                             self.properties.get('log_pseudocount', 0),
+                             'ymin')[0]
+
+        if self.properties.get('orientation', None) == 'inverted':
+            axis.set_ylim(ymax, ymin)
+        else:
+            axis.set_ylim(ymin, ymax)
+
     def __del__(self):
         return
