@@ -3,7 +3,6 @@ from pygenometracks.utilities import InputError
 from intervaltree import IntervalTree, Interval
 import matplotlib
 import numpy as np
-import matplotlib.pyplot as plt
 from matplotlib.patches import Arc, Polygon
 from .. utilities import opener, to_string, change_chrom_names, temp_file_from_intersect
 from tqdm import tqdm
@@ -207,29 +206,10 @@ file_type = {TRACK_TYPE}
         else:
             ax.set_ylim(-1, ymax)
 
-    def plot_y_axis(self, ax, plot_ax):
+    def plot_y_axis(self, axis, plot_ax):
         if self.colormap is not None and self.properties['overlay_previous'] == 'no':
             self.colormap.set_array([])
-
-            cobar = plt.colorbar(self.colormap, ax=ax, fraction=1,
-                                 orientation='vertical')
-
-            cobar.solids.set_edgecolor("face")
-            cobar.ax.tick_params(labelsize='smaller')
-            cobar.ax.yaxis.set_ticks_position('left')
-            # adjust the labels of the colorbar
-            ticks = cobar.ax.get_yticks()
-            labels = cobar.ax.set_yticklabels(ticks.astype('float32'))
-            (vmin, vmax) = cobar.mappable.get_clim()
-            for idx in np.where(ticks == vmin)[0]:
-                # if the label is at the start of the colobar
-                # move it above avoid being cut or overlapping with other track
-                labels[idx].set_verticalalignment('bottom')
-            for idx in np.where(ticks == vmax)[0]:
-                # if the label is at the end of the colobar
-                # move it a bit inside to avoid overlapping
-                # with other labels
-                labels[idx].set_verticalalignment('top')
+            GenomeTrack.plot_custom_cobar(self, axis, fraction=1)
 
     def plot_arcs(self, ax, interval):
 
