@@ -66,7 +66,7 @@ class PlotTracks(object):
     def __init__(self, tracks_file, fig_width=DEFAULT_FIGURE_WIDTH,
                  fig_height=None, fontsize=None, dpi=None,
                  track_label_width=None,
-                 plot_regions=None):
+                 plot_regions=None, plot_width=None):
         self.fig_width = fig_width
         self.fig_height = fig_height
         self.dpi = dpi
@@ -90,6 +90,13 @@ class PlotTracks(object):
                                  1 - track_label_width,
                                  track_label_width)
 
+        # Process the width:
+        if plot_width is not None:
+            self.fig_width = plot_width \
+                / (DEFAULT_MARGINS['right'] - DEFAULT_MARGINS['left']) \
+                * (1 + 2 / 3 * 0.01) \
+                / (self.width_ratios[1] / sum(self.width_ratios))
+
         font = {'size': fontsize}
         matplotlib.rc('font', **font)
         # initialize each track
@@ -367,8 +374,8 @@ class PlotTracks(object):
                 extra_keywords = [k for k in all_keywords
                                   if k not in ['file', 'type', 'line_width']]
                 if len(extra_keywords) > 0:
-                    log.warn("These parameters were specified but will not"
-                             f" be used {' '.join(extra_keywords)}.\n")
+                    log.warning("These parameters were specified but will not"
+                                f" be used {' '.join(extra_keywords)}.\n")
                 self.vlines_properties = \
                     self.check_file_exists(track_options, tracks_file_path)
                 continue
@@ -426,10 +433,10 @@ class PlotTracks(object):
                 if ' ' in name:
                     old_name = name
                     name = '_'.join(name.split(' '))
-                    log.warn(f"Deprecated Warning: The section {section_name} "
-                             f"uses parameter {old_name} but there is no more "
-                             "parameter with space in name. "
-                             f"Will be substituted by {name}.\n")
+                    log.warning(f"Deprecated Warning: The section {section_name} "
+                                f"uses parameter {old_name} but there is no more "
+                                "parameter with space in name. "
+                                f"Will be substituted by {name}.\n")
                 else:
                     old_name = name
                 # end
@@ -497,8 +504,8 @@ class PlotTracks(object):
                     unused_keys.append(name)
             # If there are unused keys they are printed in a warning.
             if len(unused_keys) > 0:
-                log.warn(f"In section {section_name}, these parameters are "
-                         f"unused:{unused_keys}.\n")
+                log.warning(f"In section {section_name}, these parameters are "
+                            f"unused:{unused_keys}.\n")
             # The track_options will be checked for the file paths:
             track_options = self.check_file_exists(track_options,
                                                    tracks_file_path,
@@ -516,8 +523,8 @@ class PlotTracks(object):
                 if track_options['overlay_previous'] == 'no' and \
                    track_options['track_class'] not in [SpacerTrack,
                                                         XAxisTrack]:
-                    log.warn("title not set for section "
-                             f"{track_options['section_name']}\n")
+                    log.warning("title not set for section "
+                                f"{track_options['section_name']}\n")
             # The track_options are added to the track_list
             track_list.append(track_options)
         # Now that they were all checked
@@ -552,9 +559,11 @@ class PlotTracks(object):
                 file_field_name = key
                 # # THIS COULD BE REMOVED IN A NEXT 1.0 VERSION
                 if file_field_name == 'boundaries_file':
-                    log.warn("The boundaries_file is not used anymore"
-                             " please use another track with the"
-                             " `overlay_previous` option.\n")
+                    log.warning("Deprecation Warning: "
+                                "The boundaries_file is not used anymore."
+                                " It will be ignored."
+                                " Please use another track with the"
+                                " `overlay_previous` option.\n")
                 # # END
                 file_names = [x for x in track_dict[file_field_name].split(" ") if x != '']
                 full_path_file_names = []
@@ -666,6 +675,22 @@ class XAxisTrack(GenomeTrack):
         super(XAxisTrack, self).__init__(*args, **kwargs)
 
     def plot(self, ax, chrom_region, region_start, region_end):
+
+        start, end = ax.get_xlim()
+        if self.properties['where'] == 'top':
+            ax.axis["x"] = ax.new_floating_axis(0, 0.2)
+            ax.axis["x"].set_axis_direction("top")
+            label_y_pos = 0.99
+            vert_align = 'top'
+        else:
+            ax.axis["x"] = ax.new_floating_axis(0, 0.9)
+            label_y_pos = 0.01
+            vert_align = 'bottom'
+
+        # First adjust the size of the label
+        ax.axis["x"].major_ticklabels.set(size=self.properties['fontsize'])
+
+        # Get the ticks values
         ticks = ax.get_xticks()
         if ticks[-1] - ticks[1] <= 1e3:
             labels = [f"{x:,.0f}"
@@ -682,23 +707,19 @@ class XAxisTrack(GenomeTrack):
                       for x in ticks]
             labels[-2] += " Mbp"
 
-        if self.properties['where'] == 'top':
-            ax.axis["x"] = ax.new_floating_axis(0, 0.2)
-            ax.axis["x"].set_axis_direction("top")
-            label_y_pos = 0.99
-            vert_align = 'top'
-        else:
-            ax.axis["x"] = ax.new_floating_axis(0, 0.9)
-            label_y_pos = 0.01
-            vert_align = 'bottom'
+        # Fix ticks values
+        ax.set_xticks(ticks)
+
+        # Fix the limits which can be overwritten by set_xticks:
+        ax.set_xlim(start, end)
+
+        # Put the label
+        ax.set_xticklabels(labels)
+
+        # Add the chromosome name
         ax.text(0.5, label_y_pos, chrom_region, horizontalalignment='center',
                 fontsize=self.properties['fontsize'],
                 verticalalignment=vert_align, transform=ax.transAxes)
 
-        ax.axis["x"].axis.set_ticklabels(labels)
-        ax.axis['x'].axis.set_tick_params(which='minor', bottom='on')
-
-        ax.axis["x"].major_ticklabels.set(size=self.properties['fontsize'])
-
     def plot_y_axis(self, ax, plot_ax):
         pass
